Attribute VB_Name = "modMain"
Option Explicit

Public Const GridSize = 20      'Each pic is 20 X 20
Public Const GridX = 27         'Grid 1,1 Top
Public Const GridY = 48         'Grid 1,1 Left
Public Const FailHX = 27        'Mission Fail Half Height
Public Const FailHY = 125       'Mission Fail Half Width

'Timer Functions
Public Declare Function SetTimer Lib "user32.dll" (ByVal hwnd As Long, ByVal nIDEvent As Long, ByVal uElapse As Long, ByVal lpTimerFunc As Long) As Long
Public Declare Function KillTimer Lib "user32.dll" (ByVal hwnd As Long, ByVal nIDEvent As Long) As Long

Public Grid(9, 9) As Integer    'Display Grid for Animation
Public MGrid(9, 9) As Integer   'Grid for markers
Public LOGrid(9, 9) As Integer  'Core Grid for mines

Public TimerID As Long          'ID to kill timer
Public Paused As Boolean        'Are we running

Public PGX As Integer           'Player Grid X
Public PGY As Integer           'Player Grid Y
Public Direction As Integer     '0=E, 1=N, 2=S, 3=W
Public CopterStp As Integer     'Copter Animation Switch
Public Marked As Integer        'Marked grid indicator
Public MarkCount As Integer     'Number of available Markers
Public IsMarked As Boolean      'Is Current Pos Marked?
Public Ticker As Long           'Second Count for play
Public TickSpread As Integer    'Counter for Timer and Ticker difference
Public CanPlay As Boolean       'Indicate if the player has started the game
Public MissionFailed As Boolean 'Did you hit a mine?
Public MissionStep As Integer   'Image stepper for Mission Fail
Public StoreStat As Integer     'Stores grid state when the copter enters

Public Sub Main()

'Load the main display
Load fMain

'Start the main loop timer
TimerID = SetTimer(fMain.hwnd, 1, 125, AddressOf TimerCallBack)

'Pause the game
Paused = True

'Create the initial display
LoadGrid

'Open the display
fMain.Show

End Sub

'Loads the initial display grid the basic tile
Public Sub LoadGrid()

Dim i As Integer
Dim j As Integer

For i = 0 To 9
    
    For j = 0 To 9
        
        BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                        ((GridSize * i) + GridY), GridSize, GridSize, _
                        fMain.pTM.hDC, 0, 0, vbSrcCopy
        
    Next
    
Next

End Sub

'Start a new game
Public Function CreateGrid() As Integer

On Error Resume Next

Dim i As Integer                'Generic counter
Dim j As Integer
Dim TempX As Integer            'Computation Grid Locatiions
Dim TempY As Integer
Dim Confirmed As Integer        'Sets the number of non-repeated grid coords

'Clear the grids
For i = 0 To 9
    
    For j = 0 To 9
        
        Grid(i, j) = 0
        
        MGrid(i, j) = 0
        
        LOGrid(i, j) = 0
        
    Next
    
Next

Debug.Print "Mine Locations :"

'Loop through and create a list of grid coords that do not repeat
Do While Confirmed < 10
    
    'New Grid Loc
    Randomize
    
    TempX = (9 * Rnd)
    
    Randomize
    
    TempY = (9 * Rnd)
    
    'Check the grid loc, if the LayOut Grid position does not contain
    'mine then put one there, if it does bypass this round and try again
    If LOGrid(TempY, TempX) = 0 Then
        
        LOGrid(TempY, TempX) = 1
        
        Debug.Print "Y: " & TempY & " X: " & TempX
        
        Confirmed = Confirmed + 1
        
    End If
Loop

Debug.Print vbCrLf

MissionFailed = False       'Reset the Mission Status

Ticker = 0                  'Reset the Ticker Count

MarkCount = 10              'Reset the Marked Count

'Reset the main display info
fMain.lInfo(0).Caption = MarkCount
fMain.lInfo(1).Caption = Ticker

'Set the direction to north
Direction = 1

'Set the startup grid position, and put the copter in that grid
PGX = 5
PGY = 5

Grid(PGY, PGX) = 1

'Set the game for use
CanPlay = True

'UnPause and start the game
Paused = False
    
End Function

'Searches the surrounding locations for mines
Public Sub SonarPing()

On Error Resume Next

Dim HitCount As Integer
Dim TempX As Integer
Dim TempY As Integer

'If the sonar spot is a mine then player is dead, and mission failed
If LOGrid(PGY, PGX) = 1 Then
    
    MissionFailed = True
    
    Exit Sub
    
End If

'Get the current grid position, and number of mines surrounding it
TempY = PGY
TempX = PGX

HitCount = PingSquare(TempY, TempX)

'Store the Mine and number of mines indicators to the grid
StoreStat = CInt("3" & CStr(HitCount))

Grid(TempY, TempX) = StoreStat

Exit Sub
ErrHandle:
End Sub

'Helper function that pings the surrounding grid spaces
Private Function PingSquare(ByVal y As Integer, ByVal x As Integer) As Integer

On Error GoTo ErrHandle

Dim HitCount As Integer
Dim ErrorCount As Integer

If LOGrid((y - 1), (x - 1)) = 1 Then HitCount = HitCount + 1

If LOGrid((y - 1), x) = 1 Then HitCount = HitCount + 1

If LOGrid((y - 1), (x + 1)) = 1 Then HitCount = HitCount + 1

If LOGrid(y, (x - 1)) = 1 Then HitCount = HitCount + 1

If LOGrid(y, (x + 1)) = 1 Then HitCount = HitCount + 1

If LOGrid((y + 1), (x - 1)) = 1 Then HitCount = HitCount + 1

If LOGrid((y + 1), x) = 1 Then HitCount = HitCount + 1

If LOGrid((y + 1), (x + 1)) = 1 Then HitCount = HitCount + 1

PingSquare = (HitCount - ErrorCount)

Exit Function
ErrHandle:
    
    'If the grid location is outside of the viewing area then
    'keep track and take those away from the count
    ErrorCount = ErrorCount + 1
    
    Resume Next
    
End Function

'Increments the values to make the copter move
Public Sub INCMove()
    
    'Set the current grid position back to the state prior to copter entry
     Grid(PGY, PGX) = StoreStat
    
    'Add the movements
    Select Case Direction
        Case 0
            'E
            
            If (PGX + 1) < 10 Then PGX = PGX + 1
                
        Case 1
            'N
            
            If (PGY - 1) > -1 Then PGY = PGY - 1
            
        Case 2
            'S
            
            If (PGY + 1) < 10 Then PGY = PGY + 1
            
        Case 3
            'W
            
            If (PGX - 1) > -1 Then PGX = PGX - 1
            
    End Select
    
    'Store the state of the position we are moving onto
    StoreStat = Grid(PGY, PGX)
    
    'Set the grid for the copter
    Grid(PGY, PGX) = 1
    
End Sub

'Set the location as marked for a mine
Public Sub SetAsMarked()

'If the grid is not previously marked and we have enough markers left then
'mark the spot
If MGrid(PGY, PGX) = 0 Then
    
    If (MarkCount - 1) > -1 Then
        
        MarkCount = MarkCount - 1
        
        MGrid(PGY, PGX) = 1
        
        'If there are no more markers left then check for the accuracy
        'of those marks
        If MarkCount = 0 Then CheckGrid
        
    End If
        
Else
    
    'The grid was previously marked so add it back and reset its coords
    MarkCount = MarkCount + 1
        
    MGrid(PGY, PGX) = 0
        
End If

End Sub

'Check the marked grid against the LayOut Grid, if any don't match
'then stop and let them work it out, if they got it, then pause the game
'and send it to end mode
Private Sub CheckGrid()

Dim i As Integer
Dim j As Integer

For i = 0 To 9
    
    For j = 0 To 9
        
        If MGrid(i, j) = 1 Then
            
            If LOGrid(i, j) <> 1 Then Exit Sub
            
        End If
        
    Next
    
Next

Paused = True

CheckWinningTime

End Sub

'Main animation loop
Public Sub TimerCallBack(ByVal hwnd As Long, ByVal uMsg As Long, ByVal idEvent As Long, _
        ByVal dwTime As Long)

Dim i As Integer
Dim j As Integer
Dim PingNumber As Integer

'is the game paused?
If Paused = False Then
    
    fMain.Cls
    
    'Copter and Sonar
    For i = 0 To 9
        
        For j = 0 To 9
            
            'Check each grid specs
            Select Case Grid(i, j)
                Case 0
                    'tile - basic animation
                    
                    BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                        ((GridSize * i) + GridY), GridSize, GridSize, _
                        fMain.pTM.hDC, 0, 0, vbSrcCopy
                
                Case 1
                    'copter - displays the copter the i,j position
                    
                    BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                        ((GridSize * i) + GridY), GridSize, GridSize, _
                        fMain.pTM.hDC, 0, 0, vbSrcCopy
                    
                    Select Case CopterStp
                        Case 0
                            
                            BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                                ((GridSize * i) + GridY), GridSize, GridSize, _
                                fMain.pCM(1).hDC, (GridSize * Direction), 0, _
                                vbSrcAnd
                            
                            BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                                ((GridSize * i) + GridY), GridSize, GridSize, _
                                fMain.pCM(0).hDC, (GridSize * Direction), 0, _
                                vbSrcPaint
                    
                            CopterStp = 1
                        
                        Case 1
                            
                            BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                                ((GridSize * i) + GridY), GridSize, GridSize, _
                                fMain.pCM(1).hDC, (GridSize * Direction), 20, _
                                vbSrcAnd
                            
                            BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                                ((GridSize * i) + GridY), GridSize, GridSize, _
                                fMain.pCM(0).hDC, (GridSize * Direction), 20, _
                                vbSrcPaint
                                
                            CopterStp = 0
                            
                    End Select
                    
                Case Is >= 30
                    'Number
                    
                    'if the spot has been sonared the
                    'display the number. The first number
                    ', always 3, indicates this is a sonared position.
                    'The next number indicates the number to display
                    PingNumber = CInt(Mid(Grid(i, j), 2, 1))
                    
                    BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                        ((GridSize * i) + GridY), GridSize, GridSize, _
                        fMain.pNM.hDC, (PingNumber * GridSize), 0, vbSrcCopy
                
            End Select
            
        Next
        
    Next
    
    'Markers
    For i = 0 To 9
        
        For j = 0 To 9
            
            'Loop through the markers and draw the markers to the grid
            If MGrid(i, j) = 1 Then
            
                Select Case Marked
                    Case 0
                        
                        BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                            ((GridSize * i) + GridY), GridSize, GridSize, _
                            fMain.pMM.hDC, 0, 0, vbSrcCopy
                    
                    Case 1
                        
                        BitBlt fMain.hDC, ((GridSize * j) + GridX), _
                            ((GridSize * i) + GridY), GridSize, GridSize, _
                            fMain.pMM.hDC, 0, 20, vbSrcCopy
                
                End Select
                
            End If
            
        Next
        
    Next
    
    'If mission failed the draw the fail message and quit the loop
    If MissionFailed = True Then
        
        'Increment the Mission Fail sequence
        If MissionStep = 1 Then
            
            BitBlt fMain.hDC, FailHX, FailHY, 200, 50, fMain.pMF.hDC, 0, 0, vbSrcAnd
            
            MissionStep = 0
            
        Else
            
            MissionStep = 1
            
        End If
        
        Exit Sub
        
    End If
    
    'Increment the Marker sequence
    If Marked = 0 Then
        
        Marked = 1
        
    Else
        
        Marked = 0
        
    End If
    
    'Increment the Ticker for timing
    If TickSpread = 8 Then
                
        Ticker = Ticker + 1
        
        TickSpread = 0
        
    Else
        
        TickSpread = TickSpread + 1
        
    End If
    
    'Set the main display
    fMain.lInfo(0).Caption = MarkCount
    
    fMain.lInfo(1).Caption = Ticker
    
    fMain.Refresh
    
End If

End Sub

